import 'package:fhir/r4.dart';
import 'package:fhir_path/fhir_path.dart';
import 'package:fhir_path/run_fhir_path.dart';

void main() {
  print(walkFhirPath(
    newResponse.toJson(),
    r"%resource.repeat(item).where(answer.exists())",
  ));
}

final newResponse = QuestionnaireResponse.fromJson({
  "resourceType": "QuestionnaireResponse",
  "item": [
    {"linkId": "/psc/preschool"},
    {
      "linkId": "/psc/preschool/irritability/nervous",
      "answer": [
        {
          "valueCoding": {
            "extension": [
              {
                "url": "http://hl7.org/fhir/StructureDefinition/ordinalValue",
                "valueDecimal": 1
              }
            ],
            "code": "Somewhat",
            "display": "Somewhat"
          }
        }
      ]
    },
    {"linkId": "/psc/preschool/irritability/sad"},
    {"linkId": "/psc/preschool/irritability/upset"},
    {"linkId": "/psc/preschool/irritability/change"},
    {"linkId": "/psc/preschool/irritability/trouble_playing"},
    {"linkId": "/psc/preschool/irritability/break_things"},
    {"linkId": "/psc/preschool/inflexibility/irritability/fights"},
    {"linkId": "/psc/preschool/inflexibility/irritability/attention"},
    {"linkId": "/psc/preschool/inflexibility/irritability/calming_down"},
    {"linkId": "/psc/preschool/irritability/one_activity"},
    {"linkId": "/psc/preschool/subscore_irritability"},
    {"linkId": "/psc/preschool/inflexibility/aggressive"},
    {"linkId": "/psc/preschool/routines/inflexibility/fidgety"},
    {"linkId": "/psc/preschool/routines/inflexibility/angry"}
  ]
});

// print(walkFhirPath(resource.toJson(), r'0.aggregate(1 + $total, 0)'));
// // JsonEncoder encoder = new JsonEncoder.withIndent('  ');
// // String prettyprint = encoder.convert(resource.toJson());
// // print(prettyprint);
// // print(walkFhirPath(resource, r'Patient.name.exists($this)'));
// // print(walkFhirPath(resource, r'Patient.name.where($this)'));
// // print(walkFhirPath(resource, r'Patient.name.given.where($this)'));

// final response = QuestionnaireResponse.fromJson(questionnaireResponse);
// print(walkFhirPath(
//     response.toJson(),
//     "(QuestionnaireResponse.item.where(linkId = '1.1').answer.valueCoding.extension.valueDecimal + "
//     "QuestionnaireResponse.item.where(linkId = '1.2').answer.valueCoding.extension.valueDecimal+ "
//     "QuestionnaireResponse.item.where(linkId = '1.3').answer.valueCoding.extension.valueDecimal) < 12"));
// print(walkFhirPath(response.toJson(),
//     r"QuestionnaireResponse.item.answer.valueCoding.extension.valueDecimal.aggregate($this + $total, 0)"));

final extPatient = Patient.fromJson({
  "resourceType": "Patient",
  "extension": [
    {
      "valueString": "1",
      "extension": [
        {
          "valueString": "4",
          "extension": [
            {"valueString": "4"}
          ]
        }
      ]
    },
    {"valueString": "2"},
    {"valueString": "3"}
  ]
});

final questionnaireResponse = {
  "resourceType": "QuestionnaireResponse",
  "id": "gcs",
  "questionnaire": "Questionnaire/gcs",
  "status": "completed",
  "subject": {
    "reference": "Patient/example",
    "display": "Peter James Chalmers"
  },
  "authored": "2014-12-11T04:44:16Z",
  "source": {"reference": "Practitioner/f007"},
  "item": [
    {
      "linkId": "1.1",
      "answer": [
        {
          "valueCoding": {
            "extension": [
              {
                "url": "http://hl7.org/fhir/StructureDefinition/ordinalValue",
                "valueDecimal": 4
              }
            ],
            "system": "http://loinc.org",
            "code": "LA6560-2",
            "display": "Confused"
          }
        }
      ]
    },
    {
      "linkId": "1.2",
      "answer": [
        {
          "valueCoding": {
            "extension": [
              {
                "url": "http://hl7.org/fhir/StructureDefinition/ordinalValue",
                "valueDecimal": 5
              }
            ],
            "system": "http://loinc.org",
            "code": "LA6566-9",
            "display": "Localizing pain"
          }
        }
      ]
    },
    {
      "linkId": "1.3",
      "answer": [
        {
          "valueCoding": {
            "extension": [
              {
                "url": "http://hl7.org/fhir/StructureDefinition/ordinalValue",
                "valueDecimal": 4
              }
            ],
            "system": "http://loinc.org",
            "code": "LA6556-0",
            "display": "Eyes open spontaneously"
          }
        }
      ]
    }
  ]
};

final resource = Patient(
  telecom: [
    ContactPoint(
        system: ContactPointSystem.email,
        use: ContactPointUse.mobile,
        rank: PositiveInt(3)),
  ],
  address: [
    Address(
      period: Period(
        extension_: [
          FhirExtension(
            valueCount: Count(unit: 'Mg'),
          ),
          FhirExtension(
            valueCount: Count(unit: 'mL'),
          ),
        ],
      ),
    ),
    Address(
      period: Period(
        extension_: [
          FhirExtension(
            extension_: [
              FhirExtension(
                extension_: [
                  FhirExtension(
                    extension_: [
                      FhirExtension(
                        valueCount: Count(unit: 'Kg'),
                      ),
                      FhirExtension(
                        valueCount: Count(unit: 'Km'),
                      ),
                    ],
                    valueCount: Count(unit: 'Kg'),
                  ),
                  FhirExtension(
                    valueCount: Count(unit: 'Km'),
                  ),
                ],
                valueCount: Count(unit: 'Kg'),
              ),
              FhirExtension(
                valueCount: Count(unit: 'Km'),
              ),
            ],
            valueCount: Count(unit: 'Kg'),
          ),
          FhirExtension(
            valueCount: Count(unit: 'Km'),
          ),
        ],
      ),
    ),
    Address(
      period: Period(
        extension_: [
          FhirExtension(
            valueCount: Count(unit: 'Feet'),
          ),
          FhirExtension(
            valueCount: Count(unit: 'inches'),
          ),
        ],
      ),
    ),
  ],
  deceasedBoolean: Boolean(false),
  name: [
    HumanName(
      use: HumanNameUse.official,
      family: 'Faulkenberry',
      given: [
        'Jason',
        'Grey',
      ],
    ),
    HumanName(
      use: HumanNameUse.official,
      family: 'Faulkenberry',
      given: [
        'Jason',
        'Grey',
      ],
    ),
    HumanName(
      family: 'Niel',
      given: [
        'Kristin',
      ],
    ),
    HumanName(
      family: 'Smith',
      given: [
        'John',
        'Jacob',
        'Jingleheimer',
      ],
    ),
  ],
);
